@@ -729,6 +729,13 @@ static struct flb_config_map config_map[] = {
      "Kubernetes authorization token file"
     },
 
+    /* Kubernetes Token command */
+    {
+     FLB_CONFIG_MAP_STR, "kube_token_command", NULL,
+     0, FLB_FALSE, 0,
+     "command to get Kubernetes authorization token"
+    },
+
     /* Include Kubernetes Labels in the final record ? */
     {
      FLB_CONFIG_MAP_BOOL, "labels", "true",
@@ -839,7 +846,12 @@ static struct flb_config_map config_map[] = {
     {
      FLB_CONFIG_MAP_TIME, "kube_meta_cache_ttl", "0",
      0, FLB_TRUE, offsetof(struct flb_kube, kube_meta_cache_ttl),
-     "configurable TTL for K8s cached metadata"
+     "configurable TTL for K8s cached metadata. " 
+     "By default, it is set to 0 which means TTL for cache entries is disabled and " 
+     "cache entries are evicted at random when capacity is reached. " 
+     "In order to enable this option, you should set the number to a time interval. " 
+     "For example, set this value to 60 or 60s and cache entries " 
+     "which have been created more than 60s will be evicted"
     },
     /* EOF */
     {0}
