@@ -278,6 +278,12 @@ static int cb_rewrite_tag_init(struct flb_filter_instance *ins,
 
     /* Register a metric to count the number of emitted records */
 #ifdef FLB_HAVE_METRICS
+    ctx->cmt_emitted = cmt_counter_create(ins->cmt,
+                                          "fluentbit", "filter", "emit_records_total",
+                                          "Total number of emitted records",
+                                          1, (char *[]) {"name"});
+
+    /* OLD api */
     flb_metrics_add(FLB_RTAG_METRIC_EMITTED,
                     "emit_records", ctx->ins->metrics);
 #endif
@@ -354,15 +360,23 @@ static int cb_rewrite_tag_filter(const void *data, size_t bytes,
     int emitted = 0;
     size_t pre = 0;
     size_t off = 0;
+#ifdef FLB_HAVE_METRICS
+    uint64_t ts;
+    char *name;
+#endif
     msgpack_sbuffer mp_sbuf;
     msgpack_packer mp_pck;
     msgpack_object map;
     msgpack_object root;
     msgpack_unpacked result;
     struct flb_rewrite_tag *ctx = (struct flb_rewrite_tag *) filter_context;
-    (void) f_ins;
     (void) config;
 
+#ifdef FLB_HAVE_METRICS
+    ts = cmt_time_now();
+    name = (char *) flb_filter_name(f_ins);
+#endif
+
     /* Create temporal msgpack buffer */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
@@ -406,6 +420,10 @@ static int cb_rewrite_tag_filter(const void *data, size_t bytes,
     }
 #ifdef FLB_HAVE_METRICS
     else if (emitted > 0) {
+        cmt_counter_add(ctx->cmt_emitted, ts, emitted,
+                        1, (char *[]) {name});
+
+        /* OLD api */
         flb_metrics_sum(FLB_RTAG_METRIC_EMITTED, emitted, ctx->ins->metrics);
     }
 #endif
