@@ -20,6 +20,7 @@
 
 #include <stdlib.h>
 #include <fluent-bit/flb_utils.h>
+#include <fluent-bit/flb_input_plugin.h>
 
 #include "fw.h"
 #include "fw_conn.h"
@@ -28,8 +29,7 @@
 struct flb_in_fw_config *fw_config_init(struct flb_input_instance *i_ins)
 {
     char tmp[16];
-    const char *buffer_size;
-    const char *chunk_size;
+    int ret = -1;
     const char *p;
     struct flb_in_fw_config *config;
 
@@ -39,11 +39,15 @@ struct flb_in_fw_config *fw_config_init(struct flb_input_instance *i_ins)
         return NULL;
     }
 
-    p = flb_input_get_property("unix_path", i_ins);
-    if (p) {
-        config->unix_path = flb_strdup(p);
+    ret = flb_input_config_map_set(i_ins, (void *)config);
+    if (ret == -1) {
+        flb_plg_error(i_ins, "config map set error");
+        flb_free(config);
+        return NULL;
     }
-    else {
+
+    p = flb_input_get_property("unix_path", i_ins);
+    if (p == NULL) {
         /* Listen interface (if not set, defaults to 0.0.0.0:24224) */
         flb_input_net_default_listener("0.0.0.0", 24224, i_ins);
         config->listen = i_ins->host.listen;
@@ -51,26 +55,6 @@ struct flb_in_fw_config *fw_config_init(struct flb_input_instance *i_ins)
         config->tcp_port = flb_strdup(tmp);
     }
 
-    /* Chunk size */
-    chunk_size = flb_input_get_property("buffer_chunk_size", i_ins);
-    if (!chunk_size) {
-        config->buffer_chunk_size = FLB_IN_FW_CHUNK_SIZE; /* 1MB */
-    }
-    else {
-        /* Convert KB unit to Bytes */
-        config->buffer_chunk_size  = flb_utils_size_to_bytes(chunk_size);
-    }
-
-    /* Buffer size */
-    buffer_size = flb_input_get_property("buffer_max_size", i_ins);
-    if (!buffer_size) {
-        config->buffer_max_size = FLB_IN_FW_CHUNK_MAX_SIZE; /* 6MB */
-    }
-    else {
-        /* Convert unit to bytes */
-        config->buffer_max_size  = flb_utils_size_to_bytes(buffer_size);
-    }
-
     if (!config->unix_path) {
         flb_debug("[in_fw] Listen='%s' TCP_Port=%s",
                   config->listen, config->tcp_port);
@@ -82,7 +66,6 @@ int fw_config_destroy(struct flb_in_fw_config *config)
 {
     if (config->unix_path) {
         unlink(config->unix_path);
-        flb_free(config->unix_path);
     }
     else {
         flb_free(config->tcp_port);
