@@ -337,7 +337,7 @@ struct flb_task *flb_task_create(uint64_t ref_id,
                                  const char *buf,
                                  size_t size,
                                  struct flb_input_instance *i_ins,
-                                 void *ic,
+                                 struct flb_input_chunk *ic,
                                  const char *tag_buf, int tag_len,
                                  struct flb_config *config,
                                  int *err)
@@ -391,6 +391,11 @@ struct flb_task *flb_task_create(uint64_t ref_id,
         o_ins = mk_list_entry(o_head,
                               struct flb_output_instance, _head);
 
+        /* skip output plugins that don't handle proper event types */
+        if (!flb_router_match_type(ic->event_type, o_ins)) {
+            continue;
+        }
+
         if (flb_routes_mask_get_bit(task_ic->routes_mask, o_ins->id) != 0) {
             route = flb_malloc(sizeof(struct flb_task_route));
             if (!route) {
